#    PGP.pm 	
#    Copyright (C) 2000 Jens Luedicke <jens@irs-net.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#######################################################################

package PMC::PGP;

sub sign_mail {
	shift;
	my ($text,$compose_edit) = @_;
	my ($i,@signed_file);
	my $mailfile = "/tmp/pmc-pgp";

        open(MAILFILE, ">$mailfile");
	print MAILFILE "$text";
        close(MAILFILE);

	&sign(
	    Textfile => "$mailfile",
	    Passphrase => $cnf_pgp_passphrase,
	);

	open(SIGNED_FILE, "$mailfile.asc");
	@signed_file = <SIGNED_FILE>;
	close(SIGNED_FILE);

	$i = $compose_edit->get_chars(0,-1);
	$compose_edit->freeze();
	$compose_edit->set_point(length($i));
	$compose_edit->backward_delete(length($i));
	$compose_edit->insert('','','',"@signed_file");
	$compose_edit->thaw();

	unlink("$mailfile");
	unlink("$mailfile.asc");
}

sub encrypt_mail {
	shift;
	my ($to,$cc,$text,$compose_edit) = @_;
	my ($recpt,$i,@encrypted_file);
	my $mailfile = "/tmp/pmc-pgp";

        open(MAILFILE, ">$mailfile");
	print MAILFILE "$text";
        close(MAILFILE);

	$recpt = $to;
	if($cc ne "") {
	    $recpt = $to.$cc;
	}

	if($recpt eq "") {
	    PMC::GUI->fyi("You did not select any PGP recipients!\n That means that To: and/or CC: are empty!");
	    return;
	} 

	&mail_sign_encrypt(
	    Textfile => "$mailfile",
	    Recipients => $recpt,
	    Userid => $cnf_pgp_userid,
	    Passphrase => $cnf_pgp_passphrase,
	);

	open(ENCRYPTED_FILE, "$mailfile.asc");
	@encrypted_file = <ENCRYPTED_FILE>;
	close(ENCRYPTED_FILE);

	$i = $compose_edit->get_chars(0,-1);
	$compose_edit->freeze();
	$compose_edit->set_point(length($i));
	$compose_edit->backward_delete(length($i));
	$compose_edit->insert('','','',"@encrypted_file");
	$compose_edit->thaw();

	unlink("$mailfile");
	unlink("$mailfile.asc");
}

sub sign_encrypt_mail {
	shift;
	my ($to,$cc,$text,$compose_edit) = @_;
	my ($recpt,$i,@encrypted_file);
	my $mailfile = "/tmp/pmc-pgp";

        open(MAILFILE, ">$mailfile") || PMC::GUI->fyi("Could not read/write $mailfile: $!");
	print MAILFILE "$text";
        close(MAILFILE);

	$recpt = $to;
	if($cc ne "") {
	    $recpt = $to.$cc;
	}

	if($recpt eq "") {
	    PMC::GUI->fyi("You did not select any PGP recipients!\n That means that To: and/or CC: are empty!");
	    return;
	} 

	&encrypt(
	    Textfile => "$mailfile",
	    Recipients => $recpt,
	    Userid => $cnf_pgp_userid,
	    Passphrase => $cnf_pgp_passphrase,
	);

	open(ENCRYPTED_FILE, "$mailfile.asc");
	@encrypted_file = <ENCRYPTED_FILE>;
	close(ENCRYPTED_FILE);

	$i = $compose_edit->get_chars(0,-1);
	$compose_edit->freeze();
	$compose_edit->set_point(length($i));
	$compose_edit->backward_delete(length($i));
	$compose_edit->insert('','','',"@encrypted_file");
	$compose_edit->thaw();

	unlink("$mailfile");
	unlink("$mailfile.asc");
}

sub decrypt_mail {
	shift;				
	    my($body) = @_;
	    my($pgp_infile,$pgp_outfile,$body_pgp); 

	    $pgp_infile = "/tmp/pmc-pgp.asc";
	    $pgp_outfile = "/tmp/pmc-pgp";	

	    open(PGP_FILE, ">$pgp_infile");
	    print PGP_FILE $body;
	    close(PGP_FILE);

	    system("perl -e 'use PMC; PMC::PGP->cleaner(Infile => $pgp_infile);'");	

	    &decrypt(
		Cypherfile => $pgp_infile,
		Outfile => $pgp_outfile,
		Passphrase => $cnf_pgp_passphrase
	    );

	    open(PGP_FILE, "$pgp_outfile");
	    $body_pgp = <PGP_FILE>;
	    close(PGP_FILE);

	    unlink("$pgp_infile");
	    unlink("$pgp_outfile");

	    my $pgp_comment = "PGP signed/encrypted mail:\n\n";
	    $body = $pgp_comment.$body_pgp;
	    return $body;
}

#### the following subs are used by the above subs ...

sub sign {
	%arg = @_;
	
	my $orig_textfile = $arg{Textfile};
	my $passphrase = $arg{Passphrase};

	system("pgp -sat -z \"$passphrase\" $orig_textfile");
}

sub encrypt {
	%arg = @_;
	
	my $orig_textfile = $arg{Textfile};
	my $recpt = $arg{Recipients};
	my $myuserid = $arg{Userid};
	
	system("pgp -ea $orig_textfile \"$recpt\" -u \"$myuserid\"");
}

sub sign_encrypt {
	%arg = @_;
	
	my $orig_textfile = $arg{Textfile};
	my $recpt = $arg{Recipients};
	my $passphrase = $arg{Passphrase};
	my $myuserid = $arg{Userid};
	
	system("pgp -esa -z \"$passphrase\" $orig_textfile \"$recpt\" -u \"$myuserid\"");
}

sub decrypt {
	%arg = @_;
	
	my $cypherfile = $arg{Cypherfile};
	my $passphrase = $arg{Passphrase};
	my $outfile = $arg{Outfile};
	
	system("pgp -z \"$passphrase\" $cypherfile -o $outfile");
}

sub cleaner {
	shift;
	%arg =  @_;
	
	$file = $arg{Infile};
	
	open(FILE, "$file");
	@lines = <FILE>;
	close(FILE);
	
	foreach $piece (@lines) {
		$piece =~ s/^\s//g;
	} 

	open(FILE, ">$file");
	print FILE @lines;
	close(FILE);
}

1;
